#property indicator_separate_window
#property indicator_buffers 1
#property indicator_plots   1
#property indicator_minimum -1
#property indicator_maximum  1

#property indicator_type1   DRAW_HISTOGRAM
#property indicator_color1  DarkTurquoise

//   
double ExtBuffer[];

//   
ENUM_TIMEFRAMES TF[5] = {PERIOD_H4, PERIOD_H1, PERIOD_M15, PERIOD_M5, PERIOD_M1};

//+------------------------------------------------------------------+
class CIndicator
  {
protected:
   string symbol;              //  
   ENUM_TIMEFRAMES timeframe;  // 

   double Open[], High[], Low[], Close[]; //   
   int BufLen; //     

public:
   //---    
   
   void Create(string sym, ENUM_TIMEFRAMES tf) {symbol = sym; timeframe = tf;};
   void Init();
   void Calculate(datetime start_time); // start_time -  ,  
  };
//+------------------------------------------------------------------+
//+   - Ichimoku
//+------------------------------------------------------------------+
class CIchimoku: public CIndicator
  {
private:
   //    
   int InpTenkan;
   int InpKijun;
   int InpSenkou;

public:   
   //   
   double ExtTenkanBuffer [1];
   double ExtKijunBuffer  [1];
   double ExtSpanABuffer  [1];
   double ExtSpanBBuffer  [1];
   double ExtChinkouBuffer[1];   

   //-------------------------------------------
   void Init(int Tenkan = 9, int Kijun = 26, int Senkou = 52)
     {
      InpTenkan = Tenkan; InpKijun = Kijun; InpSenkou = Senkou;
      BufLen = MathMax(MathMax(InpTenkan, InpKijun), InpSenkou);
     };
   //-------------------------------------------
   void Calculate(datetime start_time)
     {
/*+*/ CopyHigh (symbol,timeframe,start_time,BufLen,High);
/*+*/ CopyLow  (symbol,timeframe,start_time,BufLen,Low );
/*+*/ CopyClose(symbol,timeframe,start_time,1     ,Close);
     
//-// int limit;
      //---
//-// if(prev_calculated==0) limit=0;
//-// else                   limit=prev_calculated-1;
      //---
//-// for(int i=limit;i<rates_total;i++)
/*+*/ int i=0;
        {
         ExtChinkouBuffer[i]=Close[i];
         //--- tenkan sen
         double high=Highest(High,InpTenkan,i);
         double low=Lowest(Low,InpTenkan,i);
         ExtTenkanBuffer[i]=(high+low)/2.0;
         //--- kijun sen
         high=Highest(High,InpKijun,i);
         low=Lowest(Low,InpKijun,i);
         ExtKijunBuffer[i]=(high+low)/2.0;
         //--- senkou span a
         ExtSpanABuffer[i]=(ExtTenkanBuffer[i]+ExtKijunBuffer[i])/2.0;
         //--- senkou span b
         high=Highest(High,InpSenkou,i);
         low=Lowest(Low,InpSenkou,i);
         ExtSpanBBuffer[i]=(high+low)/2.0;
        }
      //--- done
//-// return(rates_total);     
     };
   //+------------------------------------------------------------------+
   //| get highest value for range                                      |
   //+------------------------------------------------------------------+
   double Highest(const double&array[],int range,int fromIndex)
     {
/*+*/ fromIndex=MathMax(ArraySize(array)-1, 0);     
      double res=0;
   //---
      res=array[fromIndex];
      for(int i=fromIndex;i>fromIndex-range && i>=0;i--)
        {
         if(res<array[i]) res=array[i];
        }
   //---
      return(res);
     }
   //+------------------------------------------------------------------+
   //| get lowest value for range                                       |
   //+------------------------------------------------------------------+
   double Lowest(const double&array[],int range,int fromIndex)
     {
/*+*/ fromIndex=MathMax(ArraySize(array)-1, 0);     
      double res=0;
   //---
      res=array[fromIndex];
      for(int i=fromIndex;i>fromIndex-range && i>=0;i--)
        {
         if(res>array[i]) res=array[i];
        }
   //---
      return(res);
     }     
  };
//+------------------------------------------------------------------+
//+   - Price_Channel
//+------------------------------------------------------------------+
class CChannel: public CIndicator
  {
private:
   //    
   int InpChannelPeriod;

public:   
   //   
   double ExtHighBuffer[1];
   double ExtLowBuffer [1];
   double ExtMiddBuffer[1];   

   //-------------------------------------------
   void Init(int InpChannelPeriod=22)
     {
      this.InpChannelPeriod = InpChannelPeriod;
      BufLen = InpChannelPeriod;
     };
   //-------------------------------------------
   void Calculate(datetime start_time)
     {
/*+*/ CopyHigh (symbol,timeframe,start_time,BufLen+1,High );
/*+*/ CopyLow  (symbol,timeframe,start_time,BufLen+1,Low  );
   
//-// int i,limit;
      //--- check for rates
//-// if(rates_total<InpChannelPeriod)
//-//    return(0);
      //--- preliminary calculations
//-// if(prev_calculated==0)
//-//    limit=InpChannelPeriod;
//-// else limit=prev_calculated-1;

      //--- the main loop of calculations
//-// for(i=limit;i<rates_total;i++)
/*+*/ int i=0;
        {
         ExtHighBuffer[i]=Highest(High,InpChannelPeriod,i);
         ExtLowBuffer[i]=Lowest(Low,InpChannelPeriod,i);
         ExtMiddBuffer[i]=(ExtHighBuffer[i]+ExtLowBuffer[i])/2.0;;
        }
      //--- OnCalculate done. Return new prev_calculated.
//-// return(rates_total);     
     };
  
   //+------------------------------------------------------------------+
   //| get highest value for range                                      |
   //+------------------------------------------------------------------+
   double Highest(const double &array[],int range,int fromIndex)
     {
/*+*/ fromIndex=MathMax(ArraySize(array)-2, 0);     
      double res;
      int i;
   //---
      res=array[fromIndex];
      for(i=fromIndex;i>fromIndex-range && i>=0;i--)
        {
         if(res<array[i]) res=array[i];
        }
   //---
      return(res);
     }
   //+------------------------------------------------------------------+
   //| get lowest value for range                                       |
   //+------------------------------------------------------------------+
   double Lowest(const double &array[],int range,int fromIndex)
     {
/*+*/ fromIndex=MathMax(ArraySize(array)-2, 0);     
      double res;
      int i;
   //---
      res=array[fromIndex];
      for(i=fromIndex;i>fromIndex-range && i>=0;i--)
        {
         if(res>array[i]) res=array[i];
        }
   //---
      return(res);
     }
  };
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//      
//int h_Ichimoku[5], h_Channel[5];

//    
CIchimoku o_Ichimoku[5]; CChannel o_Channel[5];

//+------------------------------------------------------------------+
void OnInit()
  {
   SetIndexBuffer(0, ExtBuffer);
   ArraySetAsSeries(ExtBuffer, true);
   
   //   
   for (int itf=0; itf<5; itf++)
     {
      o_Ichimoku[itf].Create(Symbol(), TF[itf]);
      o_Ichimoku[itf].Init(9, 26, 52);
      o_Channel [itf].Create(Symbol(), TF[itf]);
      o_Channel [itf].Init(22);
     }
  }
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[])
  {
   ArraySetAsSeries(time, true);
  
   int limit = prev_calculated ? rates_total - prev_calculated : rates_total -1;

   for (int bar = limit; bar >= 0; bar--)
     {
      //   
      datetime Time  = time [bar];
      
      //---     
      
      double Signal = 0; //  
      double bufPrice[1], bufTenkan[1], bufKijun [1], bufMid[1], bufSignal[1];
      for (int itf=0; itf<5; itf++)
        {
         //===  

         CopyClose(Symbol(), TF[itf], Time, 1, bufPrice);
         double Price = bufPrice[0];

         //===  
         
         o_Ichimoku[itf].Calculate(Time);
         //CopyBuffer(h_Ichimoku[itf], 0, Time, 1, bufTenkan);
         //double Tenkan = bufTenkan[0];
         double Tenkan = o_Ichimoku[itf].ExtTenkanBuffer[0];
         //CopyBuffer(h_Ichimoku[itf], 1, Time, 1, bufKijun );    
         //double Kijun  = bufKijun [0];
         double Kijun  = o_Ichimoku[itf].ExtKijunBuffer [0];
           
         if (Tenkan > Kijun) Signal++;
         if (Tenkan < Kijun) Signal--;
          
         //===  
         
         o_Channel[itf].Calculate(Time);
         //CopyBuffer(h_Channel [itf], 2, Time, 1, bufMid);
         //double Mid = bufMid[0];
         double Mid = o_Channel[itf].ExtMiddBuffer[0];

         if (Price > Mid) Signal++;
         if (Price < Mid) Signal--;
        }
        
      ExtBuffer[bar] = Signal/10;
     }

   return(rates_total);
  }
//+------------------------------------------------------------------+